
/* Network traffic analyzer
 * (c) 2001, Emil Mikulic.
 */

/* accounting for protocols */
#include "proto.h"



i64	proto_tcp_in,  proto_tcp_out,
	proto_udp_in,  proto_udp_out,
	proto_icmp_in, proto_icmp_out;



inline void proto_transfer_in(const byte proto, const dword amount)
{
	switch (proto)
	{
	case 6: add_64bit(proto_tcp_in, amount); break;
	case 17: add_64bit(proto_udp_in, amount); break;
	case 1: add_64bit(proto_icmp_in, amount); break;
	default: freakout("They fed me a bad protocol!");
	}
}



inline void proto_transfer_out(const byte proto, const dword amount)
{
	switch (proto)
	{
	case 6: add_64bit(proto_tcp_out, amount); break;
	case 17: add_64bit(proto_udp_out, amount); break;
	case 1: add_64bit(proto_icmp_out, amount); break;
	default: freakout("They fed me a bad protocol!");
	}
}



void proto_db_dump(FILE *fp)
{
	fprintf(fp, "%u:%u|%u:%u\n%u:%u|%u:%u\n%u:%u|%u:%u\n",
			HI(proto_tcp_in),
			LO(proto_tcp_in),
			HI(proto_tcp_out),
			LO(proto_tcp_out),
			HI(proto_udp_in),
			LO(proto_udp_in),
			HI(proto_udp_out),
			LO(proto_udp_out),
			HI(proto_icmp_in),
			LO(proto_icmp_in),
			HI(proto_icmp_out),
			LO(proto_icmp_out));
}



inline char *proto_name(const byte protocol)
{
	/* full list in: netinet/in.h */
	switch (protocol) {
		case 0: return "TCP Dummy protocol";
		case 1: return "Internet Control Message Protocol";
		case 2: return "Internet Group Management Protocol";
		case 4: return "IPIP Tunnel";
		case 94: return "IPIP Tunnel (older KA9Q variety)";
		case 6: return "Transmission Control Protocol";
		case 8: return "Exterior Gateway Protocol";
		case 12: return "PUP Protocol";
		case 17: return "User Datagram Protocol";
		case 22: return "XNS IDP Protocol";
		case 29: return "SO Transport Protocol Class 4";
		case 41: return "IPv6 header";
		case 43: return "IPv6 routing header";
		case 44: return "IPv6 fragmentation header";
		case 46: return "Reservation Protocol";
		case 47: return "General Routing Encapsulation";
		case 50: return "Encapsulating Security Payload";
		case 51: return "Authentication Header";
		case 58: return "ICMPv6";
		case 59: return "IPv6 no next header";
		case 60: return "IPv6 destination options";
		case 92: return "Multicast Transport Protocol";
		case 98: return "Encapsulation Header";
		case 103: return "Protocol Independent Multicast";
		case 108: return "Compression Header Protocol";
		case 255: return "Raw IP packet";
	}
	return "&nbsp;";
}



inline char *proto_name_short(const byte protocol)
{
	/* full list in: netinet/in.h */
	switch (protocol) {
		case 0: return "Dummy";
		case 1: return "ICMP";
		case 2: return "IGMP";
		case 4: return "IPIP";
		case 94: return "IPIP-KA9Q";
		case 6: return "TCP";
		case 8: return "EGP";
		case 12: return "PUP";
		case 17: return "UDP";
		case 22: return "XNS-IDP";
		case 58: return "ICMPv6";
		case 103: return "PImcast";
		case 255: return "Raw";
	}
	return "UNKNOWN";
}

